use cfgfile;

include "../pkg/npcs/npcinfo";
include "include/itemtypes";

///////////////////////////
// GetBaseArmor will caclulate how much armor (spell buffs and shield armor excluded)
// the player has.
///////////////////////////
function getArmor(who)
	return Cint(getBaseArmor(who) + getArmorMod(who));
endfunction

///////////////////////////
// GetArmor will calculate how much armor (shield armor excluded) a player has
///////////////////////////
function getBaseArmor(who)
	var armor := 0;
	if(who.acctname)
		armor := CINT((who.ar - getVisualShieldArmor(who)) - getArmorMod(who));
	else
		armor := GetNpcTemplateArmor(who);
	endif
	return armor;
endfunction

///////////////////////////
// GetArmorMod shows the armor that isn't BaseArmor (from items such as platemail)
// This includes both mage armor and other.
//
// however it does not account for armor mods that aren't included in this .inc!
///////////////////////////
function getArmorMod(who)
	return getBonusArmor(who) + getMageArmor(who);
endfunction

///////////////////////////
// GetVisualShieldArmor calculates how much shield armor is displayed on the 
// status bar for the specified player or npc.
///////////////////////////
function getVisualShieldArmor(who)
	var shieldar := getShieldArmor(who);
	if(shieldar > 0)
		var parry := GetAttribute (who, ATTRIBUTEID_PARRY);
		return CalculateVisualShieldArmor(parry, shieldar);
	endif

	return 0;
endfunction

///////////////////////////
// GetVisualShieldArmor calculates how much shield armor is displayed on the 
// status bar.
///////////////////////////
function CalculateVisualShieldArmor(parry_skill, shield_armor)
	var armor := CINT(CDbl(shield_armor) * (CDbl(parry_skill) / 200));
	if(armor < 1)
		armor:= 1;
	endif
	return armor;
endfunction

////////////////////////////
// GetShieldArmor gets the amount of armor that the targets shield has.
// if he/she doesn't have a shield it will return 0
////////////////////////////
function getShieldArmor(who)
	var shield := GetEquipmentByLayer (who, LAYER_HAND2);
	if (shield && IsShield (shield))
		return shield.ar;
	endif

	return 0;
endfunction

////////////////////////////
// GetBaseShieldArmor gets the amount of armor that the targets shield has,
// if we don't count any magical additions.
// if he/she doesn't have a shield it will return 0
////////////////////////////
function getBaseShieldArmor(who)
	var shield := GetEquipmentByLayer (who, LAYER_HAND2);
	if (shield && IsShield (shield))
		return shield.ar-shield.ar_mod;
	endif

	return 0;
endfunction

////////////////////////////
// This function gets the amount of armor that the specified targets
// 'should' have.
////////////////////////////
function CalcArmorFromItems(who)
	var zone_ar_rating := 0;
	// get the armor information
	var armorfile := ReadConfigFile (":armor:itemdesc");

	var hitzones := {}; //making a array of all hitzones
	hitzones.append("Hands");	
	hitzones.append("Neck");
	hitzones.append("Legs/feet");
	hitzones.append("Head");
	hitzones.append("Arms");
	hitzones.append("Body");
	
	//armor values for each hitzone
	var handarmor := 0;
	var neckarmor := 0;
	var legarmor := 0;
	var armarmor := 0;
	var chestarmor := 0;
	var headarmor := 0;

	var armor_itemdesc_elem, coverage_array;
	foreach item in ListEquippedItems (who)
		armor_itemdesc_elem := FindConfigElem (armorfile, item.objtype);
		if (armor_itemdesc_elem)
			coverage_array := GetConfigStringArray (armor_itemdesc_elem, "Coverage");
			if (len (coverage_array)) //makes sure the item is a armor piece
				foreach zone in hitzones
					if (zone in coverage_array)		
						var armorcount := CINT(armor_itemdesc_elem.AR + item.ar_mod);
						if(item.quality)
							armorcount := CINT(armorcount * item.quality);
						endif
	
						//checking if the items gives more armor to it's zone...crude code but it works
						if ( zone == "Hands" && armorcount > handarmor)
							handarmor := armorcount;
						elseif ( zone == "Neck" && armorcount > neckarmor)
							neckarmor := armorcount;
						elseif ( zone == "Legs/feet" && armorcount > legarmor)
							legarmor := armorcount;
						elseif ( zone == "Arms" && armorcount > armarmor)
							armarmor := armorcount;
						elseif ( zone == "Head" && armorcount > headarmor)
							headarmor := armorcount;
						elseif ( zone == "Body" && armorcount > chestarmor)
							chestarmor := armorcount;
						endif	
					endif
				endforeach
			endif
		endif
	endforeach


	//calculating the average armor amount
	zone_ar_rating := zone_ar_rating + (chestarmor * 0.44);
	zone_ar_rating := zone_ar_rating + (headarmor * 0.14);
	zone_ar_rating := zone_ar_rating + (armarmor * 0.14);
	zone_ar_rating := zone_ar_rating + (legarmor * 0.14);
	zone_ar_rating := zone_ar_rating + (handarmor * 0.07);
	zone_ar_rating := zone_ar_rating + (neckarmor * 0.07);
	zone_ar_rating := CINT(zone_ar_rating); // the value has to be a int!
	//zone ar rating now shows the same value as the status bar, with the exception of shields and special buff armor
	
	return zone_ar_rating;
endfunction

///////////////////////////////////
// Get the amount of base armor that a npc has (accoording to its template)
// if no value is found (for whatever reason) then the armor count of the gear will
// be used.
///////////////////////////////////
function GetNpcTemplateArmor(who)
	if(!who.npctemplate)
		return 0;
	endif

	var defender_elem := GetNpcTemplateElem (who.npctemplate);
	if(!defender_elem) //this SHOULDN'T happen, but better save then sorry
		return CalcArmorFromItems(who);
	endif

	var armor := GetConfigInt (defender_elem, "AR");
	if(!armor || armor == 0)
		return CalcArmorFromItems(who);
	endif		

	return armor;
endfunction

///////////////////////////////////
// Get the maximum amount armor that you 'can' get from mage buffs
///////////////////////////////////
function getMageArmorLimit(who)
	return CINT(getObjProperty(who, "#mage_armor_limit"));
endfunction

///////////////////////////////////
// Get the current amount of armor that you 'can' get from mage buffs
///////////////////////////////////
function getTotalMageArmor(who)
	var marmor := getObjProperty(who, "#total_mage_armor");
	if(!marmor)
		marmor := 0;
	endif
	return CINT(marmor);
endfunction

function getMageArmor(who)
	var marmor := getObjProperty(who, "#mage_armor");
	if(!marmor)
		marmor := 0;
	endif
	return CINT(marmor);
endfunction

///////////////////////////////////
// increase the current amount of mage armor
///////////////////////////////////
function increaseMageArmor(who, amount)
	set_critical(1);
	var armor := getTotalMageArmor(who);

	setObjProperty(who, "#total_mage_armor", CINT(armor+amount));
	recalcMageArmor(who);
	set_critical(0);
endfunction

///////////////////////////////////
// Decrease the current amount of mage armor
///////////////////////////////////
function decreaseMageArmor(who, amount)
	set_critical(1);
	var armor := getTotalMageArmor(who);

	setObjProperty(who, "#total_mage_armor", CINT(armor - amount));
	recalcMageArmor(who);
	set_critical(0);
endfunction

///////////////////////////////////
// Recalc the maximum amount of mage armor that you 'can' get from mage buffs
///////////////////////////////////
function RecalcMageArmorLimit(who, aitem := 0, add := 0)
	set_critical(1);
	var climit := getMageArmorLimit(who);
	var limit := 0;

	// get the armor information
	var armorcfgfile := ReadConfigFile (":armor:itemdesc");

	var hitzones := {}; //making a array of all hitzones
	hitzones.append("Hands");	
	hitzones.append("Neck");
	hitzones.append("Legs/feet");
	hitzones.append("Head");
	hitzones.append("Arms");
	hitzones.append("Body");
	
	//armor values for each hitzone
	var handarmor := 40;
	var neckarmor := 40;
	var legarmor := 40;
	var armarmor := 40;
	var chestarmor := 40;
	var headarmor := 40;

	var items := ListEquippedItems (who);
	if(aitem)
		if(add)
			items.append(aitem);
		else
			var oldItems := items;
			items := array;
			foreach item in oldItems
				if(item.serial != aitem.serial)
					items.append(item);				
				endif
			endforeach
		endif
	endif	

	var armor_elem, check, coverage_array;
	foreach item in items
			armor_elem := FindConfigElem (armorcfgfile, item.objtype);
			if (armor_elem)
				coverage_array := GetConfigStringArray (armor_elem, "Coverage");
				if (len (coverage_array)) //makes sure the item is a armor piece
					foreach zone in hitzones
						if (zone in coverage_array)		
							check := GetConfigInt (armor_elem, "BUFF");
							if (check)
								//checking if the items gives more armor to it's zone...crude code but it works
								if ( zone == "Hands" && check < handarmor)
									handarmor := check;
								elseif ( zone == "Neck" && check < neckarmor)
									neckarmor := check;
								elseif ( zone == "Legs/feet" && check < legarmor)
									legarmor := check;
								elseif ( zone == "Arms" && check < armarmor)
									armarmor := check;
								elseif ( zone == "Head" && check < headarmor)
									headarmor := check;
								elseif ( zone == "Body" && check < chestarmor)
									chestarmor := check;
								endif	
							endif
						endif
					endforeach
				endif
			endif
	endforeach

	limit := CDbl(limit);
	limit := limit + (CDbl(chestarmor) * 0.44);
	limit := limit + (CDbl(headarmor) * 0.14);
	limit := limit + (CDbl(armarmor) * 0.14);
	limit := limit + (CDbl(legarmor) * 0.14);
	limit := limit + (CDbl(handarmor) * 0.07);
	limit := limit + (CDbl(neckarmor) * 0.07);
	limit := CINT(limit); // the value has to be a int!

	setObjProperty(who, "#mage_armor_limit", limit);

	recalcMageArmor(who);
	set_critical(0);
	return limit;
endfunction

function recalcMageArmor(who)
	set_critical(1);
	var limit := getMageArmorLimit(who);
	var carmor := getMageArmor(who);
	var tarmor := getTotalMageArmor(who);
	if(tarmor > limit)
		tarmor := limit;
	endif

	who.ar_mod := who.ar_mod - carmor; //remove the old
	who.ar_mod := who.ar_mod + tarmor; //add the new, I'm such a noob...
	setObjProperty(who, "#mage_armor", tarmor);
	set_critical(0);
	return tarmor;	
endfunction

///////////////////////////////////
// Get the amount of none mage armor buff
///////////////////////////////////
function getBonusArmor(who)
	return CINT(getObjProperty(who, "#bonus_armor"));
endfunction

///////////////////////////////////
// Increase the amount of none mage armor buff
///////////////////////////////////
function increaseArmor(who, amount)
	set_critical(1);
	var armor := getObjProperty(who, "#bonus_armor");
	setObjProperty(who, "#bonus_armor", CINT(armor+amount));
	who.ar_mod := who.ar_mod + amount;
	set_critical(0);
endfunction

///////////////////////////////////
// Decrease the amount of none mage armor buff
///////////////////////////////////
function decreaseArmor(who, amount)
	set_critical(1);
	var armor := getObjProperty(who, "#bonus_armor");
	setObjProperty(who, "#bonus_armor", CINT(armor-amount));
	who.ar_mod := who.ar_mod - amount;
	set_critical(0);
endfunction

function DoArmorHitDamage(defender)
	var armorfile := ReadConfigFile (":armor:itemdesc");

	//Figure out which zone got hit, so we can apply damage to it
	var hitzone := "";
	var therandomnumber := random (100);
	if (therandomnumber < 7)
		hitzone := "Hands";
	elseif (therandomnumber < 14)
		hitzone := "Neck";
	elseif (therandomnumber < 28)
		hitzone := "Legs/feet";
	elseif (therandomnumber < 42)
		hitzone := "Head";
	elseif (therandomnumber < 56)
		hitzone := "Arms";
	else
		hitzone := "Body";
	endif

	var zone_ar_rating := 0;
	var armor;
	var armor_elem, coverage_array;
	foreach item in ListEquippedItems (defender)
		armor_elem := FindConfigElem (armorfile, item.objtype);
		if (armor_elem)
			coverage_array := GetConfigStringArray (armor_elem, "Coverage");
			if (len (coverage_array)) //makes sure the item is a armor piece
				if (hitzone in coverage_array)		
					var armorcount := CINT(armor_elem.AR + item.ar_mod);
					if(item.quality)
						armorcount := CINT(armorcount * item.quality);
					endif

					//Do some damage to the armor, it's on the right zone ;)
					if (random (1000) < 4)
						item.hp := item.hp - 2;
						if (item.hp < 1 || item.hp > 1000)
							SendSysMessage (defender, "Your " + item.desc + " breaks!");
							DestroyItem (item);
						endif
					endif

					if(item)
						if(zone_ar_rating < armorcount)
							zone_ar_rating := armorcount;
							armor := item;
						endif
					endif
				endif
			endif
		endif
	endforeach

	if (armor)
		if (random (180) == 1)
			armor.hp := armor.hp - 1;
			if (armor.hp < 1 || armor.hp > 1000)
				SendSysMessage (defender, "Your " + armor.desc + " breaks!");
				DestroyItem (armor);
			endif
		endif
	endif
endfunction